from typing import List, Any, Dict

import pandas as pd
from enums.CatalogConfigurationEnum import CatalogConfigurationEnum
from enums.FileTypeEnum import FileTypeEnum

import logging

logger = logging.getLogger()


def get_type_file(key: str) -> FileTypeEnum:
    result = FileTypeEnum.EXCEL
    try:
        file_type_sufix = key.rfind(".")
        file_type = key[file_type_sufix+1:]
        result = FileTypeEnum(file_type)
    except Exception as e:
        logger.error(f"Error obteniedo el tipo de archivo de {key}. {e}")
    return result


def get_modified_prefix(prefix: str) -> str:
    try:
        if prefix == "/":
            prefix = ""
        elif not prefix.endswith("/") and prefix != "":
            prefix += "/"
    except Exception as e:
        logger.error(f"Error modificando prefijo de {prefix}. {e}")
    finally:
        return prefix


def add_period_to_sufix(name: str, period: str) -> str:
    result = name
    try:
        position = name.rfind(".")
        result = name[:position] + period + name[position:]
    except Exception as e:
        logger.error(f"Error añadiendo periodo al nombre del archivo {name}. {e}")
    return result


def remove_invalid_rows(df: pd.DataFrame, valid_cols: List[str]) -> pd.DataFrame:
    try:
        df = df.dropna(how='all', subset=valid_cols)
    except Exception as e:
        logger.error(f"Error removiendo filas inválidas. {e}")
    finally:
        return df


def remove_fields(df: pd.DataFrame, fields_omitted: List[str]) -> pd.DataFrame:
    try:
        if len(fields_omitted) > 0:
            df = df.loc[:, ~df.columns.isin(fields_omitted)]
    except Exception as e:
        logger.error(f"Error removiendo columnas. {e}")
    finally:
        return df


def update_dict_with_catalogs(data_dict: Dict[str, Any], data: Dict[str, Any], catalog_name: str,
                              default_prefix: str) -> Dict[str, Any]:
    try:
        catalog = data[catalog_name]
        catalog_type = catalog["type"]
        catalog_prefix = catalog["prefix"]
        if catalog_type == CatalogConfigurationEnum.CATALOGO.value:
            catalog_prefix = get_modified_prefix(catalog_prefix)
        else:
            catalog_prefix = default_prefix
        s3_catalog = catalog_prefix + catalog["pattern"]
        data_dict.update({'s3_'+catalog_name: s3_catalog, catalog_name+'_key': catalog["key_field"],
                          catalog_name+'_value': catalog["value_field"]})
        if "delimiter" in catalog.keys():
            data_dict.update({catalog_name+'_delimiter': catalog["delimiter"]})
    except Exception as e:
        logger.error(f"Error actualizando dict de catalogos. {e}")
    finally:
        return data_dict
